# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from .M30 import BTMinerM30V10, BTMinerM30V20
from .M30K import BTMinerM30KV10
from .M30L import BTMinerM30LV10
from .M30S import (
    BTMinerM30SV10,
    BTMinerM30SV20,
    BTMinerM30SV30,
    BTMinerM30SV40,
    BTMinerM30SV50,
    BTMinerM30SV60,
    BTMinerM30SV70,
    BTMinerM30SV80,
    BTMinerM30SVE10,
    BTMinerM30SVE20,
    BTMinerM30SVE30,
    BTMinerM30SVE40,
    BTMinerM30SVE50,
    BTMinerM30SVE60,
    BTMinerM30SVE70,
    BTMinerM30SVF10,
    BTMinerM30SVF20,
    BTMinerM30SVF30,
    BTMinerM30SVG10,
    BTMinerM30SVG20,
    BTMinerM30SVG30,
    BTMinerM30SVG40,
    BTMinerM30SVH10,
    BTMinerM30SVH20,
    BTMinerM30SVH30,
    BTMinerM30SVH40,
    BTMinerM30SVH50,
    BTMinerM30SVH60,
    BTMinerM30SVI20,
)
from .M30S_Plus import (
    BTMinerM30SPlusV10,
    BTMinerM30SPlusV20,
    BTMinerM30SPlusV30,
    BTMinerM30SPlusV40,
    BTMinerM30SPlusV50,
    BTMinerM30SPlusV60,
    BTMinerM30SPlusV70,
    BTMinerM30SPlusV80,
    BTMinerM30SPlusV90,
    BTMinerM30SPlusV100,
    BTMinerM30SPlusVE30,
    BTMinerM30SPlusVE40,
    BTMinerM30SPlusVE50,
    BTMinerM30SPlusVE60,
    BTMinerM30SPlusVE70,
    BTMinerM30SPlusVE80,
    BTMinerM30SPlusVE90,
    BTMinerM30SPlusVE100,
    BTMinerM30SPlusVF20,
    BTMinerM30SPlusVF30,
    BTMinerM30SPlusVG20,
    BTMinerM30SPlusVG30,
    BTMinerM30SPlusVG40,
    BTMinerM30SPlusVG50,
    BTMinerM30SPlusVG60,
    BTMinerM30SPlusVH10,
    BTMinerM30SPlusVH20,
    BTMinerM30SPlusVH30,
    BTMinerM30SPlusVH40,
    BTMinerM30SPlusVH50,
    BTMinerM30SPlusVH60,
)
from .M30S_Plus_Plus import (
    BTMinerM30SPlusPlusV10,
    BTMinerM30SPlusPlusV20,
    BTMinerM30SPlusPlusVE30,
    BTMinerM30SPlusPlusVE40,
    BTMinerM30SPlusPlusVE50,
    BTMinerM30SPlusPlusVF40,
    BTMinerM30SPlusPlusVG30,
    BTMinerM30SPlusPlusVG40,
    BTMinerM30SPlusPlusVG50,
    BTMinerM30SPlusPlusVH10,
    BTMinerM30SPlusPlusVH20,
    BTMinerM30SPlusPlusVH30,
    BTMinerM30SPlusPlusVH40,
    BTMinerM30SPlusPlusVH50,
    BTMinerM30SPlusPlusVH60,
    BTMinerM30SPlusPlusVH70,
    BTMinerM30SPlusPlusVH80,
    BTMinerM30SPlusPlusVH90,
    BTMinerM30SPlusPlusVH100,
    BTMinerM30SPlusPlusVJ20,
    BTMinerM30SPlusPlusVJ30,
)
from .M31 import BTMinerM31V10, BTMinerM31V20
from .M31H import BTMinerM31HV10, BTMinerM31HV40
from .M31L import BTMinerM31LV10
from .M31S import (
    BTMinerM31SV10,
    BTMinerM31SV20,
    BTMinerM31SV30,
    BTMinerM31SV40,
    BTMinerM31SV50,
    BTMinerM31SV60,
    BTMinerM31SV70,
    BTMinerM31SV80,
    BTMinerM31SV90,
    BTMinerM31SVE10,
    BTMinerM31SVE20,
    BTMinerM31SVE30,
)
from .M31S_Plus import (
    BTMinerM31SPlusV10,
    BTMinerM31SPlusV20,
    BTMinerM31SPlusV30,
    BTMinerM31SPlusV40,
    BTMinerM31SPlusV50,
    BTMinerM31SPlusV60,
    BTMinerM31SPlusV80,
    BTMinerM31SPlusV90,
    BTMinerM31SPlusV100,
    BTMinerM31SPlusVE10,
    BTMinerM31SPlusVE20,
    BTMinerM31SPlusVE30,
    BTMinerM31SPlusVE40,
    BTMinerM31SPlusVE50,
    BTMinerM31SPlusVE60,
    BTMinerM31SPlusVE80,
    BTMinerM31SPlusVF20,
    BTMinerM31SPlusVF30,
    BTMinerM31SPlusVG20,
    BTMinerM31SPlusVG30,
)
from .M31SE import BTMinerM31SEV10, BTMinerM31SEV20, BTMinerM31SEV30
from .M32 import BTMinerM32V10, BTMinerM32V20
from .M32S import BTMinerM32S
from .M33 import BTMinerM33V10, BTMinerM33V20, BTMinerM33V30
from .M33S import BTMinerM33SVG30
from .M33S_Plus import BTMinerM33SPlusVG20, BTMinerM33SPlusVH20, BTMinerM33SPlusVH30
from .M33S_Plus_Plus import (
    BTMinerM33SPlusPlusVG40,
    BTMinerM33SPlusPlusVH20,
    BTMinerM33SPlusPlusVH30,
)
from .M34S_Plus import BTMinerM34SPlusVE10
from .M36S import BTMinerM36SVE10
from .M36S_Plus import BTMinerM36SPlusVG30
from .M36S_Plus_Plus import BTMinerM36SPlusPlusVH30
from .M39 import BTMinerM39V10, BTMinerM39V20, BTMinerM39V30
